/*---------------------------------------------------------------------------*/

/*!  \file  integrator.c
     \brief  integrate the Langevin equation numerically.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/

/*  DO NOT EDIT THIS FILE. This file is generated from integrator.c.m4.  */

#include "common.h"

#ifdef HAVE_MATH_H
#include <math.h>
#endif

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

#include "vector.h"
#include "particles.h"
#include "force.h"
#include "celllist.h"
#include "integrator.h"

/*---------------------------------------------------------------------------*/
static void integrate_langevin_equation2_srk_first(soft_particles2 *particles,
                                                     box_geometry *geometry,double dt);
static void integrate_langevin_equation2_srk_second(soft_particles2 *particles,
                                                      box_geometry *geometry,double dt);

static void integrate_langevin_equation3_srk_first(soft_particles3 *particles,
                                                     box_geometry *geometry,double dt);
static void integrate_langevin_equation3_srk_second(soft_particles3 *particles,
                                                      box_geometry *geometry,double dt);

static void calculate_gamma_dot(box_geometry *geometry,double dt);
static void integrate_dynamic_equation_geometry(box_geometry *geometry,double dt);

/*---------------------------------------------------------------------------*/
void integrate_langevin_equation(soft_particles *particles,
                                 linked_cell_list *celllist,
                                 polymer_blend *blend,
                                 box_geometry *geometry,double dt)
{
    calculate_gamma_dot(geometry,dt);
    clear_interaction_force(particles);
    generate_random_force(particles);


        /*  the first step of the stochastic Runge-Kutta method  */
    construct_linked_cell_list(celllist,particles,geometry);
    accumulate_site_interaction_force(particles,celllist,blend,geometry);
    accumulate_tethering_interaction_force(particles,blend,geometry);

        switch(particles->dimension)
    {
        case 2:
            integrate_langevin_equation2_srk_first(&(particles->particles2),geometry,dt);
            break;

        case 3:
            integrate_langevin_equation3_srk_first(&(particles->particles3),geometry,dt);
            break;

        default:
            break;
    }


    integrate_dynamic_equation_geometry(geometry,dt);
    set_particle_positions_star_into_box(particles,geometry);


        /*  the second step of the stochastic Runge-Kutta method  */
    construct_linked_cell_list_star(celllist,particles,geometry);
    accumulate_site_interaction_force_star(particles,celllist,blend,geometry);
    accumulate_tethering_interaction_force_star(particles,blend,geometry);

        switch(particles->dimension)
    {
        case 2:
            integrate_langevin_equation2_srk_second(&(particles->particles2),geometry,dt);
            break;

        case 3:
            integrate_langevin_equation3_srk_second(&(particles->particles3),geometry,dt);
            break;

        default:
            break;
    }


    set_particle_positions_into_box(particles,geometry);
}

/*---------------------------------------------------------------------------*/
void calculate_gamma_dot(box_geometry *geometry,double dt)
{
    if(geometry->omega != 0.0)
    {
            /*  calculate the effective strain rate  */
        geometry->gamma_dot = geometry->kappa / geometry->omega / dt
                * (sin(geometry->theta + geometry->omega * dt)
                   - sin(geometry->theta));
    }
    else
    {
        geometry->gamma_dot = geometry->kappa;
    }
}

/*---------------------------------------------------------------------------*/
void integrate_dynamic_equation_geometry(box_geometry *geometry,double dt)
{
    if(geometry->kappa != 0.0)
    {
        geometry->delta_l += dt * geometry->gamma_dot * geometry->l[1];
        geometry->theta += geometry->omega * dt;

        while(geometry->delta_l < 0.0)
            geometry->delta_l += geometry->l[0];
        while(geometry->delta_l >= geometry->l[0])
            geometry->delta_l -= geometry->l[0];

        while(geometry->theta >= 2.0 * M_PI)
            geometry->theta -= 2.0 * M_PI;
    }
}

/*---------------------------------------------------------------------------*/
void integrate_langevin_equation2_srk_first(soft_particles2 *particles,
                                              box_geometry *geometry,double dt)
{
    int i,j;

    if(geometry->gamma_dot == 0.0)
    {
            /*  no shear flow  */
        for(i = 0;i < particles->n;i++)
        {
            for(j = 0;j < particles->m[i];j++)
            {
                particles->r_star[i][j][0] = particles->r[i][j][0]
                          + dt * particles->f[i][j][0]
                          + sqrt(2.0 * dt) * particles->w[i][j][0];
particles->r_star[i][j][1] = particles->r[i][j][1]
                          + dt * particles->f[i][j][1]
                          + sqrt(2.0 * dt) * particles->w[i][j][1];
            }
        }
    }
    else
    {
            /*  shear flow  */
        for(i = 0;i < particles->n;i++)
        {
            for(j = 0;j < particles->m[i];j++)
            {
                particles->r_star[i][j][0] = particles->r[i][j][0]
                          + dt * geometry->gamma_dot * particles->r[i][j][1]
                          + dt * particles->f[i][j][0]
                          + sqrt(2.0 * dt) * particles->w[i][j][0];
particles->r_star[i][j][1] = particles->r[i][j][1]
                          
                          + dt * particles->f[i][j][1]
                          + sqrt(2.0 * dt) * particles->w[i][j][1];
            }
        }
    }
}

/*---------------------------------------------------------------------------*/
void integrate_langevin_equation2_srk_second(soft_particles2 *particles,
                                               box_geometry *geometry,double dt)
{
    int i,j;

    if(geometry->gamma_dot == 0.0)
    {
            /*  no shear flow  */
        for(i = 0;i < particles->n;i++)
        {
            for(j = 0;j < particles->m[i];j++)
            {
                particles->r[i][j][0] +=
                          + 0.5 * dt * (particles->f[i][j][0] + particles->f_star[i][j][0])
                          + sqrt(2.0 * dt) * particles->w[i][j][0];
particles->r[i][j][1] +=
                          + 0.5 * dt * (particles->f[i][j][1] + particles->f_star[i][j][1])
                          + sqrt(2.0 * dt) * particles->w[i][j][1];
            }
        }
    }
    else
    {
            /*  shear flow  */
        for(i = 0;i < particles->n;i++)
        {
            for(j = 0;j < particles->m[i];j++)
            {
                particles->r[i][j][0] +=
                          + dt * geometry->gamma_dot * particles->r[i][j][1]
                          + 0.5 * dt * (particles->f[i][j][0] + particles->f_star[i][j][0])
                          + sqrt(2.0 * dt) * particles->w[i][j][0];
particles->r[i][j][1] +=
                          
                          + 0.5 * dt * (particles->f[i][j][1] + particles->f_star[i][j][1])
                          + sqrt(2.0 * dt) * particles->w[i][j][1];
            }
        }
    }
}

/*---------------------------------------------------------------------------*/
void integrate_langevin_equation3_srk_first(soft_particles3 *particles,
                                              box_geometry *geometry,double dt)
{
    int i,j;

    if(geometry->gamma_dot == 0.0)
    {
            /*  no shear flow  */
        for(i = 0;i < particles->n;i++)
        {
            for(j = 0;j < particles->m[i];j++)
            {
                particles->r_star[i][j][0] = particles->r[i][j][0]
                          + dt * particles->f[i][j][0]
                          + sqrt(2.0 * dt) * particles->w[i][j][0];
particles->r_star[i][j][1] = particles->r[i][j][1]
                          + dt * particles->f[i][j][1]
                          + sqrt(2.0 * dt) * particles->w[i][j][1];
particles->r_star[i][j][2] = particles->r[i][j][2]
                          + dt * particles->f[i][j][2]
                          + sqrt(2.0 * dt) * particles->w[i][j][2];
            }
        }
    }
    else
    {
            /*  shear flow  */
        for(i = 0;i < particles->n;i++)
        {
            for(j = 0;j < particles->m[i];j++)
            {
                particles->r_star[i][j][0] = particles->r[i][j][0]
                          + dt * geometry->gamma_dot * particles->r[i][j][1]
                          + dt * particles->f[i][j][0]
                          + sqrt(2.0 * dt) * particles->w[i][j][0];
particles->r_star[i][j][1] = particles->r[i][j][1]
                          
                          + dt * particles->f[i][j][1]
                          + sqrt(2.0 * dt) * particles->w[i][j][1];
particles->r_star[i][j][2] = particles->r[i][j][2]
                          
                          + dt * particles->f[i][j][2]
                          + sqrt(2.0 * dt) * particles->w[i][j][2];
            }
        }
    }
}

/*---------------------------------------------------------------------------*/
void integrate_langevin_equation3_srk_second(soft_particles3 *particles,
                                               box_geometry *geometry,double dt)
{
    int i,j;

    if(geometry->gamma_dot == 0.0)
    {
            /*  no shear flow  */
        for(i = 0;i < particles->n;i++)
        {
            for(j = 0;j < particles->m[i];j++)
            {
                particles->r[i][j][0] +=
                          + 0.5 * dt * (particles->f[i][j][0] + particles->f_star[i][j][0])
                          + sqrt(2.0 * dt) * particles->w[i][j][0];
particles->r[i][j][1] +=
                          + 0.5 * dt * (particles->f[i][j][1] + particles->f_star[i][j][1])
                          + sqrt(2.0 * dt) * particles->w[i][j][1];
particles->r[i][j][2] +=
                          + 0.5 * dt * (particles->f[i][j][2] + particles->f_star[i][j][2])
                          + sqrt(2.0 * dt) * particles->w[i][j][2];
            }
        }
    }
    else
    {
            /*  shear flow  */
        for(i = 0;i < particles->n;i++)
        {
            for(j = 0;j < particles->m[i];j++)
            {
                particles->r[i][j][0] +=
                          + dt * geometry->gamma_dot * particles->r[i][j][1]
                          + 0.5 * dt * (particles->f[i][j][0] + particles->f_star[i][j][0])
                          + sqrt(2.0 * dt) * particles->w[i][j][0];
particles->r[i][j][1] +=
                          
                          + 0.5 * dt * (particles->f[i][j][1] + particles->f_star[i][j][1])
                          + sqrt(2.0 * dt) * particles->w[i][j][1];
particles->r[i][j][2] +=
                          
                          + 0.5 * dt * (particles->f[i][j][2] + particles->f_star[i][j][2])
                          + sqrt(2.0 * dt) * particles->w[i][j][2];
            }
        }
    }
}

/*---------------------------------------------------------------------------*/
