/*---------------------------------------------------------------------------*/

/*!  \file  luaread.c
     \brief  read input Lua file for the simulator  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2008 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_LUA_H
#include <lua.h>
#endif
#ifdef HAVE_LUALIB_H
#include <lualib.h>
#endif
#ifdef HAVE_LAUXLIB_H
#include <lauxlib.h>
#endif

#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#include "xmalloc.h"
#include "luautil.h"
#include "luaread.h"

/*---------------------------------------------------------------------------*/
typedef struct original_monomer_data
{
    int nm;
    char **name;
    double **epsilon;
} original_monomer_data;

typedef struct original_polymer_data
{
    int ns;
    char **monomer;
} original_polymer_data;

/*---------------------------------------------------------------------------*/
static simulation_condition *read_simulation_condition_lua(lua_State *L);
static simulation_iofiles *read_simulation_iofiles_lua(lua_State *L,
                                              simulation_condition *condition);
static box_geometry *read_box_geometry_lua(lua_State *L);
static polymer_blend *read_polymer_blend_lua(lua_State *L);
static original_monomer_data *read_original_monomer_data(lua_State *L);
static original_polymer_data *read_original_polymer_data(lua_State *L,
                                                         char *polymer_name);
static original_monomer_data *allocate_original_monomer_data(void);
static void free_original_monomer_data(original_monomer_data *monomer);
static original_polymer_data *allocate_original_polymer_data(void);
static void free_original_polymer_data(original_polymer_data *polymer);
static polymer_blend *allocate_polymer_blend_using_original_data(int np,
                                                         char **polymer_name,
                                                                 int *number,
                                              original_monomer_data *monomer,
                                              original_polymer_data **polymer);
static double get_epsilon_from_monomer_name(char *name1,char *name2,
                                            original_monomer_data *monomer);

/*---------------------------------------------------------------------------*/
void read_input_lua(char *input_lua,simulation_condition **condition,
                    simulation_iofiles **iofiles,box_geometry **geometry,
                    polymer_blend **blend)
{
    lua_State *L;

    L = luau_open_file(input_lua);
    if(L == NULL)
    {
        exit(EXIT_FAILURE);
    }

    *condition = read_simulation_condition_lua(L);
    *iofiles = read_simulation_iofiles_lua(L,*condition);
    *geometry = read_box_geometry_lua(L);
    *blend = read_polymer_blend_lua(L);

    lua_close(L);
}

/*---------------------------------------------------------------------------*/
simulation_condition *read_simulation_condition_lua(lua_State *L)
{
    simulation_condition *condition;

    condition = allocate_simulation_condition();

    luau_push_table_or_exit(L,"condition");


    luau_get_bool_or_exit(L,"save_position_sequential",
                          &condition->save_position_sequential);
/*     luau_get_bool_or_exit(L,"save_force_sequential", */
/*                           &condition->save_force_sequential); */
    luau_get_bool_or_exit(L,"save_stress_sequential",
                          &condition->save_stress_sequential);
    luau_get_bool_or_exit(L,"save_energy_sequential",
                          &condition->save_energy_sequential);
    luau_get_bool_or_exit(L,"save_geometry_sequential",
                          &condition->save_geometry_sequential);
    luau_get_bool_or_exit(L,"save_dx_sequential",
                          &condition->save_dx_sequential);
    luau_get_bool_or_exit(L,"apply_shear",&condition->apply_shear);

    luau_get_unsigned_long_or_exit(L,"seed",&condition->seed);
    luau_get_int_or_exit(L,"iteration_max",&condition->iteration_max);
    luau_get_int_or_exit(L,"interval",&condition->interval);
    luau_get_double_or_exit(L,"dt",&condition->dt);
    if(condition->apply_shear)
    {
        luau_get_double_or_exit(L,"kappa",&condition->kappa);
        luau_get_double_or_exit(L,"omega",&condition->omega);
    }


    luau_pop_table(L);

    return condition;
}

/*---------------------------------------------------------------------------*/
simulation_iofiles *read_simulation_iofiles_lua(lua_State *L,
                                              simulation_condition *condition)
{
    simulation_iofiles *iofiles;

    iofiles = allocate_simulation_iofiles();

    luau_push_table_or_exit(L,"file");


    if(condition->save_position_sequential)
        luau_get_string_or_exit(L,"position_template",&iofiles->position_template);
    else
        luau_get_string_or_exit(L,"position_output",&iofiles->position_output);
/*     if(condition->save_force_sequential) */
/*         luau_get_string_or_exit(L,"force_template",&iofiles->force_template); */
/*     else */
/*         luau_get_string_or_exit(L,"force_output",&iofiles->force_output); */
    if(condition->save_stress_sequential)
        luau_get_string_or_exit(L,"stress_template",&iofiles->stress_template);
    else
        luau_get_string_or_exit(L,"stress_output",&iofiles->stress_output);
    if(condition->save_energy_sequential)
        luau_get_string_or_exit(L,"energy_template",&iofiles->energy_template);
    else
        luau_get_string_or_exit(L,"energy_output",&iofiles->energy_output);
    if(condition->save_geometry_sequential)
        luau_get_string_or_exit(L,"geometry_template",&iofiles->geometry_template);
    else
        luau_get_string_or_exit(L,"geometry_output",&iofiles->geometry_output);
    if(condition->save_dx_sequential)
        luau_get_string_or_exit(L,"dx_template",&iofiles->dx_template);
    else
        luau_get_string_or_exit(L,"dx_output",&iofiles->dx_output);


    luau_pop_table_or_exit(L);

    return iofiles;
}

/*---------------------------------------------------------------------------*/
box_geometry *read_box_geometry_lua(lua_State *L)
{
    box_geometry *geometry;
    double *l;
    int n;

    geometry = allocate_box_geometry();

    luau_push_table_or_exit(L,"geometry");


    luau_get_int_or_exit(L,"dimension",&geometry->dimension);

    n = luau_get_double_array_or_exit(L,"l",&l);
    if(n < geometry->dimension)
    {
        fprintf(stderr,
                "`geometry->l' has only %d values while the dimension is %d.",
                n,geometry->dimension);
        exit(EXIT_FAILURE);
    }
    geometry->l[0] = l[0];
    geometry->l[1] = ((geometry->dimension >= 2) ? (l[1]) : (1));
    geometry->l[2] = ((geometry->dimension >= 3) ? (l[2]) : (1));

    luau_free_double_array(l);

    luau_pop_table_or_exit(L);

    return geometry;
}

/*---------------------------------------------------------------------------*/
polymer_blend *read_polymer_blend_lua(lua_State *L)
{
    polymer_blend *blend;
    original_monomer_data *monomer;
    original_polymer_data **polymer;
    char **polymer_name;
    int *number;
    int i,np;

    luau_push_table_or_exit(L,"blend");

    np = luau_get_string_array_or_exit(L,"polymer",&polymer_name);
    luau_get_int_array_or_exit(L,"number",&number);

    luau_pop_table_or_exit(L);


    monomer = read_original_monomer_data(L);
    polymer = xmalloc(sizeof(original_polymer_data) * np);
    for(i = 0;i < np;i++)
        polymer[i] = read_original_polymer_data(L,polymer_name[i]);

    blend = allocate_polymer_blend_using_original_data(np,polymer_name,number,
                                                       monomer,polymer);

    free_original_monomer_data(monomer);
    for(i = 0;i < np;i++)
        free_original_polymer_data(polymer[i]);
    free(polymer);

    luau_free_int_array(number);
    luau_free_string_array(polymer_name);

    return blend;
}

/*---------------------------------------------------------------------------*/
original_monomer_data *read_original_monomer_data(lua_State *L)
{
    int i,j;
    original_monomer_data *monomer;

    monomer = allocate_original_monomer_data();

    luau_push_table_or_exit(L,"monomer");


    monomer->nm = luau_get_string_array_or_exit(L,"name",&monomer->name);
    luau_get_double_array2d_or_exit(L,"epsilon",&monomer->epsilon);

    luau_pop_table_or_exit(L);

        /*  symmetrize the interaction parameter matrix.  */
    for(i = 0;i < monomer->nm;i++)
    {
        for(j = 0;j <= i;j++)
        {
            monomer->epsilon[i][j] = monomer->epsilon[j][i];
        }
    }

    return monomer;
}

/*---------------------------------------------------------------------------*/
original_polymer_data *read_original_polymer_data(lua_State *L,
                                                  char *polymer_name)
{
    original_polymer_data *polymer;

    polymer = allocate_original_polymer_data();

    luau_push_table_or_exit(L,polymer_name);

    polymer->ns = luau_get_string_array_or_exit(L,"monomer",&polymer->monomer);

    luau_pop_table_or_exit(L);

    return polymer;
}

/*---------------------------------------------------------------------------*/
original_monomer_data *allocate_original_monomer_data(void)
{
    original_monomer_data *monomer;

    monomer = xmalloc(sizeof(original_monomer_data));

    return monomer;
}

/*---------------------------------------------------------------------------*/
void free_original_monomer_data(original_monomer_data *monomer)
{
    if(monomer == NULL)
        return;

    luau_free_string_array(monomer->name);
    luau_free_double_array2d(monomer->epsilon);

    free(monomer);
}

/*---------------------------------------------------------------------------*/
original_polymer_data *allocate_original_polymer_data(void)
{
    original_polymer_data *polymer;

    polymer = xmalloc(sizeof(original_monomer_data));

    return polymer;
}

/*---------------------------------------------------------------------------*/
void free_original_polymer_data(original_polymer_data *polymer)
{
    if(polymer == NULL)
        return;

    luau_free_string_array(polymer->monomer);

    free(polymer);
}

/*---------------------------------------------------------------------------*/
polymer_blend *allocate_polymer_blend_using_original_data(int np,
                                                         char **polymer_name,
                                                                 int *number,
                                              original_monomer_data *monomer,
                                              original_polymer_data **polymer)
{
    int i,j,k,i2,j2,k2;
    int ns;
    polymer_blend *blend;

    ns = 0;
    for(i = 0;i < np;i++)
    {
        if((polymer[i]->ns <= 0) || (polymer[i]->ns >= 3))
        {
            fprintf(stderr,"the number of subchains of `%s' is %d.\ncurrently only 1 subchain or 2 subchains are supported.\n",
                    polymer_name[i],polymer[i]->ns);
            exit(EXIT_FAILURE);
        }
        
        ns += polymer[i]->ns;
    }

    blend = allocate_polymer_blend(ns);

    k = 0;
    for(i = 0;i < np;i++)
    {
        for(j = 0;j < polymer[i]->ns;j++)
        {
            blend->m[k] = number[i];
            blend->next[k] = (((j + 1) < polymer[i]->ns) ? k + 1 : -1);

            k2 = 0;
            for(i2 = 0;i2 < np;i2++)
            {
                for(j2 = 0;j2 < polymer[i2]->ns;j2++)
                {
                    blend->epsilon[k][k2] = get_epsilon_from_monomer_name(
                        polymer[i]->monomer[j],polymer[i2]->monomer[j2],
                        monomer);

                    k2++;
                }
            }

            k++;
        }
    }

    return blend;
}

/*---------------------------------------------------------------------------*/
double get_epsilon_from_monomer_name(char *name1,char *name2,
                                     original_monomer_data *monomer)
{
    int i,j;

    for(i = 0;i < monomer->nm;i++)
    {
        if(strcmp(name1,monomer->name[i]) == 0)
        {
            for(j = 0;j < monomer->nm;j++)
            {
                if(strcmp(name2,monomer->name[j]) == 0)
                    return monomer->epsilon[i][j];
            }
        }
    }

    return 0.0;
}

/*---------------------------------------------------------------------------*/
