/*---------------------------------------------------------------------------*/

/*!  \file  particles.h
     \brief  the soft particles which represent polymers.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2008 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_PARTICLES_H
#define INCLUDE_PARTICLES_H

#include "vector.h"
#include "tensor.h"
#include "blend.h"
#include "geometry.h"

/*---------------------------------------------------------------------------*/
/*!  \brief  positions of particles and foreces acting on particles in 2d systems.  */
typedef struct soft_particles2
{
        /*!  \brief  number of particle species.  */
    int n;
        /*!  \brief  number of particles.  */
    int *m;

        /*!  \brief  positions of particles.  */
    vector2 **r;
        /*!  \brief  thermodynamic and potential forces acting on particles.  */
    vector2 **f;
        /*!  \brief  random forces acting on particles.  */
    vector2 **w;

        /*!  \brief  positions of particles at the middle point.  */
    vector2 **r_star;
        /*!  \brief  forces acting on particles at the middle point.  */
    vector2 **f_star;

        /*!  \brief  stress tensor.  */
    symmetric_tensor2 **sigma;
} soft_particles2;

/*!  \brief  positions of particles and foreces acting on particles in 3d systems.  */
typedef struct soft_particles3
{
        /*!  \brief  number of particle species.  */
    int n;
        /*!  \brief  number of particles.  */
    int *m;

        /*!  \brief  positions of particles.  */
    vector3 **r;
        /*!  \brief  thermodynamic and potential forces acting on particles.  */
    vector3 **f;
        /*!  \brief  random forces acting on particles.  */
    vector3 **w;

        /*!  \brief  positions of particles at the middle point.  */
    vector3 **r_star;
        /*!  \brief  forces acting on particles at the middle point.  */
    vector3 **f_star;

        /*!  \brief  stress tensor.  */
    symmetric_tensor3 **sigma;
} soft_particles3;

/*!  \brief  positions of particles and foreces acting on particles.  */
typedef struct soft_particles
{
        /*  \brief  dimension of the system.  */
    int dimension;

        /*!  \brief  particles in 2d systems.  */
    soft_particles2 particles2;
        /*!  \brief  particles in 3d systems.  */
    soft_particles3 particles3;
} soft_particles;
/* typedef struct soft_particles soft_particles; */

/*---------------------------------------------------------------------------*/
/*!  \brief  allocate new soft particles.
     \param  blend  polymer blend
     \param  geometry  geometry of the simulation box  */
soft_particles *allocate_soft_particles(polymer_blend *blend,
                                        box_geometry *geometry);
/*!  \brief  free soft particles.
     \param  particles  soft particles to free  */
void free_soft_particles(soft_particles *particles);
/*   \brief  set particle positions randomly.
     \param  particles  soft particles
     \param  blend  information of the polymer blend
     \param  geometry  geometry of the simulation box  */
void set_particle_positions_random(soft_particles *particles,
                                   polymer_blend *blend,
                                   box_geometry *geometry);
/*   \brief  set particles which positions are out of the box into the box.
     \param  particles  soft particles
     \param  geometry  geometry of the simulation box  */
void set_particle_positions_into_box(soft_particles *particles,
                                     box_geometry *geometry);
void set_particle_positions_star_into_box(soft_particles *particles,
                                          box_geometry *geometry);

/*   \brief  get total number of soft particles in the system.
     \param  particles  soft particles */
int total_number_of_soft_particles(soft_particles *particles);
int total_number_of_soft_particles2(soft_particles2 *particles);
int total_number_of_soft_particles3(soft_particles3 *particles);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
