/*---------------------------------------------------------------------------*/

/*!  \file  luaread.c
     \brief  read input Lua file for the simulator  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2008 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_LUA_H
#include <lua.h>
#endif
#ifdef HAVE_LUALIB_H
#include <lualib.h>
#endif
#ifdef HAVE_LAUXLIB_H
#include <lauxlib.h>
#endif

#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#include "xmalloc.h"
#include "luautil.h"
#include "luaread.h"

/*---------------------------------------------------------------------------*/
static void read_parameters_lua(lua_State *L,int *dimension,int *m,double *dt,
                                int *save_stress_sequential,
                                char **stress_template,
                                int *save_geometry_sequential,
                                char **geometry_output);
static void read_number_of_particles_lua(lua_State *L,int *n);
static int get_number_of_monomers_in_polymer(lua_State *L,char *polymer);

/*---------------------------------------------------------------------------*/
void read_input_lua(char *input_lua,int *dimension,int *n,int *m,double *dt,
                    int *save_stress_sequential,char **stress_template,
                    int *save_geometry_sequential,char **geometry_output)
{
    lua_State *L;

    L = luau_open_file(input_lua);
    if(L == NULL)
    {
        exit(EXIT_FAILURE);
    }

    read_parameters_lua(L,dimension,m,dt,
                        save_stress_sequential,stress_template,
                        save_geometry_sequential,geometry_output);
    read_number_of_particles_lua(L,n);

    lua_close(L);
}

/*---------------------------------------------------------------------------*/
void read_parameters_lua(lua_State *L,int *dimension,int *m,double *dt,
                         int *save_stress_sequential,char **stress_template,
                         int *save_geometry_sequential,char **geometry_output)
{
    int iteration_max,interval;
    double dt_orig;

    luau_push_table_or_exit(L,"condition");

    luau_get_bool_or_exit(L,"save_stress_sequential",
                          save_stress_sequential);
    luau_get_bool_or_exit(L,"save_geometry_sequential",
                          save_geometry_sequential);
    luau_get_int_or_exit(L,"iteration_max",
                         &iteration_max);
    luau_get_int_or_exit(L,"interval",
                         &interval);
    luau_get_double_or_exit(L,"dt",
                            &dt_orig);

    luau_pop_table(L);

    *m = iteration_max / interval;
    *dt = dt_orig * interval;


    luau_push_table_or_exit(L,"file");

    luau_get_string_or_exit(L,"stress_template",stress_template);
    luau_get_string_or_exit(L,"geometry_output",geometry_output);

    luau_pop_table(L);


    luau_push_table_or_exit(L,"geometry");

    luau_get_bool_or_exit(L,"dimension",
                          dimension);

    luau_pop_table(L);
}

/*---------------------------------------------------------------------------*/
void read_number_of_particles_lua(lua_State *L,int *n)
{
    int np;
    int i;
    char **polymer;
    int *number;

    luau_push_table_or_exit(L,"blend");

    np = luau_get_string_array_or_exit(L,"polymer",&polymer);
    luau_get_int_array_or_exit(L,"number",&number);

    luau_pop_table(L);


    *n = 0;
    for(i = 0;i < np;i++)
    {
        *n += get_number_of_monomers_in_polymer(L,polymer[i]) * number[i];
    }

    free(*polymer);
    free(polymer);
    free(number);
}

/*---------------------------------------------------------------------------*/
static int get_number_of_monomers_in_polymer(lua_State *L,char *polymer)
{
    int n;
    char **monomer;

    luau_push_table_or_exit(L,polymer);

    n = luau_get_string_array_or_exit(L,"monomer",&monomer);

    luau_pop_table_or_exit(L);

    free(*monomer);
    free(monomer);

    return n;
}

/*---------------------------------------------------------------------------*/
