/*---------------------------------------------------------------------------*/

/*!  \file  relaxation.c
     \brief  calculate the stress relaxation function.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2008 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include <stdio.h>

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include "xmalloc.h"
#include "relaxation.h"

/*---------------------------------------------------------------------------*/
double **calculate_stress_relaxation_moduli(double **sigma,int dimension,
                                            double volume,int m)
{
    int i,j,k;
    int n_elements;
    double **g;
    double p;

    n_elements = (dimension * (dimension + 1)) / 2;

    g = xmalloc(sizeof(double *) * m);
    *g = xmalloc(sizeof(double) * m * n_elements);

    for(i = 1;i < m;i++)
    {
        g[i] = g[i - 1] + n_elements;
    }

        /*  subtract the pressure (the diagonal part).  */
    p = 0;
    for(i = 0;i < m;i++)
    {
        for(k = 0;k < dimension;k++)
        {
            p += sigma[i][k];
        }
    }
    p /= (m * dimension);

    for(i = 0;i < m;i++)
    {
        for(k = 0;k < dimension;k++)
        {
            sigma[i][k] -= p;
        }
    }

        /*  calculate the shear relaxation moduli by using the
            Green-Kubo formula. */
    for(i = 0;i < m;i++)
    {
        for(k = 0;k < n_elements;k++)
        {
            g[i][k] = 0;
        }

        for(j = 0;j < (m - i);j++)
        {
            for(k = 0;k < n_elements;k++)
            {
                g[i][k] += sigma[j][k] * sigma[j + i][k];
            }
        }

        for(k = 0;k < n_elements;k++)
        {
            g[i][k] *= volume / (m - i);
        }
    }

    return g;
}

/*---------------------------------------------------------------------------*/
void write_stress_relaxation_moduli_text(char *filename,double **g,
                                         int dimension,int m,double dt)
{
    int i,j;
    FILE *fp;

    if((filename == NULL) || (strlen(filename) == 0))
    {
        fp = stdout;
    }
    else
    {
        fp = fopen(filename,"wt");
    }

    for(i = 0;i < m;i++)
    {
        fprintf(fp,"%f ",dt * i);

        for(j = 0;j < ((dimension * (dimension + 1)) / 2);j++)
        {
            fprintf(fp,"%f ",g[i][j]);
        }
        fprintf(fp,"\n");
    }

    if(fp != stdout)
    {
        fclose(fp);
    }
}

/*---------------------------------------------------------------------------*/
