/*---------------------------------------------------------------------------*/

/*!  \file  drops-stress2relaxation.c
     \brief  utility to calculate the stress relaxation function.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2008 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0
#endif
#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#include "getopt.h"
#include "luaread.h"
#include "textread.h"
#include "relaxation.h"

/*---------------------------------------------------------------------------*/
static char *programstring = "spreads-stress2relaxation";

static struct option longopts[] =
{
    {"input", required_argument, NULL, 'i'},
    {"output", required_argument, NULL, 'o'},
    {"version", no_argument, NULL, 'v'},
    {"help", no_argument, NULL, 'h'},
    {0, 0, NULL, 0}
};

static char *optstring = "i:o:vh";

static char *helpstring =
"\
Usage: %s [options] [input]\n\
Options:\n\
  -i, --input <file>       use <file> as the simulation input file\n\
  -o, --output <file>      write the output stress relaxation to <file>\n\
  -v, --version            output version information and exit\n\
  -h, --help               display this help and exit\n\
";

static char *copyingstring = COPYRIGHT_STRING;

/*---------------------------------------------------------------------------*/
static int process_option(int argc,char **argv,char **input,char **output);
static double calculate_system_volume(int dimension,double *l);

/*---------------------------------------------------------------------------*/
int main(int argc,char **argv)
{
    char *input = DEFAULT_INPUT_FILE;
    char *output = NULL;
    double **sigma;
    double **g;
    double l[3],delta_l;
    double volume;
    int dimension,n,m;
    double dt;
    int save_stress_sequential,save_geometry_sequential;
    char *stress_template,*geometry_output;

    if(process_option(argc,argv,&input,&output) != 0)
    {
        return EXIT_FAILURE;
    }

    read_input_lua(input,&dimension,&n,&m,&dt,
                   &save_stress_sequential,&stress_template,
                   &save_geometry_sequential,&geometry_output);

    if(!save_stress_sequential)
    {
        fprintf(stderr,"the required output stress files are not saved correctly.\n");
        return EXIT_FAILURE;
    }
    if(save_geometry_sequential)
    {
        fprintf(stderr,"sequentially saved geometry file is not supported yet.\n");
        return EXIT_FAILURE;
    }

    read_geometry_text(geometry_output,&dimension,l,&delta_l);
    read_stress_text_sequential(stress_template,&sigma,dimension,m);

    volume = calculate_system_volume(dimension,l);
    g = calculate_stress_relaxation_moduli(sigma,dimension,volume,m);

    write_stress_relaxation_moduli_text(output,g,dimension,m,dt);

    free(*sigma);
    free(sigma);
    free(*g);
    free(g);

    return EXIT_SUCCESS;
}

/*---------------------------------------------------------------------------*/
int process_option(int argc,char **argv,char **input,char **output)
{
    int c;
    int option_index;
    char *program_name;

    program_name = argv[0];

    do
    {
        option_index = 0;
        c = getopt_long(argc,argv,optstring,longopts,&option_index);

        switch(c)
        {
            case 'i':
                    /*  input file  */
                *input = optarg;
                break;

            case 'o':
                    /*  output file  */
                *output = optarg;
                break;

            case 'v':
                    /*  version  */
                fprintf(stderr,copyingstring,programstring,PACKAGE,VERSION);
                exit(EXIT_SUCCESS);
                break;

            case 'h':
                    /*  help  */
                fprintf(stderr,helpstring,program_name);
                exit(EXIT_SUCCESS);
                break;

            case '?':
                    /*  invalid option  */
                exit(EXIT_FAILURE);
                break;

            case -1:
                break;

            default:
                fprintf(stderr,
                        "error: getopt returned character code 0%o\n", c);
                return 1;
        }
    }
    while(c != -1);


    if(optind < argc)
    {
            /*  input file  */
        *input = argv[optind++];
    }

    return 0;
}

/*---------------------------------------------------------------------------*/
double calculate_system_volume(int dimension,double *l)
{
    int i;
    double v;

    v = 1;

    for(i = 0;i < dimension;i++)
    {
        v *= l[i];
    }

    return v;
}

/*---------------------------------------------------------------------------*/
